/*
 *  2.5.        `[', `]' 
 *  .    .
 */

#include <stdio.h>
#include <string.h>

#define STACK_MEM_CHUNK     16


const char *strClosingParen=")}]";
const char *strOpeningParen="({[";

const char *strErrOpenFile          ="error: cannont open file `%s'\n";
const char *strErrMissingClosing    ="error: missing `%c'\n";
const char *strErrTooManyClosing    ="L%ld-C%ld: too many `%c'\n";
const char *strErrMismatchingClosing="L%ld-C%ld: mismatching `%c'\n";

int  PopAndCheckParen(int ClosingID, unsigned long line, unsigned long col);

int  Push(int Value);
int  Pop(int *Value);
void FreeStack(void);


int main(int argc, char **argv)
{
  int c, id, ret;
  unsigned long line, col;

  /* If user provided filename, try to redirect stdin to that file */
  if ( argc > 1 ) {
    if ( !freopen(*(argv+1), "r", stdin) ) {
      fprintf(stderr, strErrOpenFile, *(argv+1));
      return -1;
    }
  }

  /* Main loop */
  ret = 0;
  line = 1;
  col = 0;
  while ( !ret && (c=getchar()) != EOF ) {

    /* Keep track of line and column */
    if ( c == '\n' ) {
      line++;
      col = 0;
    } else
      col++;

    switch ( c ) {
    case '(':
      Push(0);
      break;
    case ')':
      ret = PopAndCheckParen(0, line, col);
      break;
    case '{':
      Push(1);
      break;
    case '}':
      ret = PopAndCheckParen(1, line, col);
      break;
    case '[':
      Push(2);
      break;
    case ']':
      ret = PopAndCheckParen(2, line, col);
      break;
    default:
      break;
    }
  }

  if ( !ret ) {
    if ( Pop(&id) ) {
      fprintf(stderr, strErrMissingClosing, *(strClosingParen + id));
      ret = -1;
    } else
      fputs("all parentheses matched\n", stdout);
  }

  FreeStack();

  return ret;
}

int PopAndCheckParen(int ClosingID, unsigned long line, unsigned long col)
{
  int PoppedID;

  if ( !Pop(&PoppedID) ) {
    fprintf(stderr, strErrTooManyClosing, 
            line, col, *(strClosingParen + ClosingID));
    return -1;
  }
  if ( PoppedID != ClosingID ) {
    fprintf(stderr, strErrMismatchingClosing, 
            line, col, *(strClosingParen+ClosingID));
    return -1;
  }

  return 0;
}



static int *Stack;
static int StackTop = -1;
static int StackLen = 0;

/* Returns length of stack on success, 0 on failure to grow stack */
int Push(int Value)
{
  int *ptr;

  /* Grow stack if necessary */
  if ( ++StackTop >= StackLen ) {
    StackLen += STACK_MEM_CHUNK;
    if ( !(ptr = (int *) realloc(Stack, StackLen * sizeof(*Stack))) ) {
      StackLen -= STACK_MEM_CHUNK;
      return 0;
    }
    Stack = ptr;
  }
  /* Push the value onto stack */
  *(Stack + StackTop) = Value;

  return StackTop + 1;
}

/* Returns -1 on success, 0 if stack is empty */
int Pop(int *Value)
{
  if ( StackTop >= 0 ) {
    *Value = *(Stack + StackTop--);
    return -1;
  }

  return 0;
}

void FreeStack(void)
{
  free(Stack);
  Stack = NULL;
  StackLen = 0;
  StackTop = -1;
}



/*
 * Local variables:
 * compile-command: "gcc 2_05.c && ./a.out 2_05.in"
 * End:
 */
