/*
 *  3.5.       N>2,   
 * ,  :
 *
 *    A_1 * A_N, A_2 * A_(N-1), ..., A_[(N+1)/2] * A_[N/2+1]
 */

#include <stdio.h>
#include <stdlib.h>

typedef struct _item {
  int ItemKey;
  int ItemValue;
  struct _item *ptrNext;
} item, *itemptr;

itemptr MakeUpList(int n);
int    *CalculateResult(itemptr ptrHead, int n);
void    PrintOut(itemptr ptrHead, int *ptrResult, int n);
void    FreeListMem(itemptr ptrHead);



int main()
{
  int n;
  itemptr ptrHead;
  int *ptrResult;

  /* Seed the random number generator */
  srand(time(0));

  /* Get list length */
  fputs("Enter list length: ", stdout);
  scanf("%d", &n);

  /* Make up a list with arbitrary contents */
  if ( !(ptrHead = MakeUpList(n)) ) {
    puts("Error allocating memory for the list.");
    return -1;
  }

  /* Allocate memory for the result buffer */
  if ( !(ptrResult = CalculateResult(ptrHead, n)) ) {
    puts("Error allocating memory for the result.");
    FreeListMem(ptrHead);
    return -1;
  }

  /* Print out the list and the result */
  PrintOut(ptrHead, ptrResult, n);

  /* Free the memory taken up by the list and result buffer */
  FreeListMem(ptrHead);
  free(ptrResult);

  /* Bail out */
  return 0;
}



itemptr MakeUpList(int n)
{
  int i;
  itemptr ptrHead, ptrPrev, ptrCurr;

  ptrHead = (itemptr) malloc(sizeof(item));
  if ( !ptrHead )
    return NULL;
  ptrHead->ItemKey = 0;
  ptrHead->ItemValue = rand() / (RAND_MAX/10);
  ptrPrev = ptrHead;

  for ( i = 1; i < n; i++ ) {
    ptrCurr = (itemptr) malloc(sizeof(item));
    if ( !ptrCurr ) {
      ptrPrev->ptrNext = NULL;
      FreeListMem(ptrHead);
      return NULL;
    }
    ptrCurr->ItemKey = i;
    /* Make sure the values are sane */
    ptrCurr->ItemValue = rand() / (RAND_MAX/10);
    ptrPrev->ptrNext = ptrCurr;
    ptrPrev = ptrCurr;
  }
  ptrPrev->ptrNext = NULL;

  return ptrHead;
}



int *CalculateResult(itemptr ptrHead, int n)
{
  int i, tmp;
  itemptr ptrCurr;
  int *ptrResult;

  ptrResult = (int *) malloc( sizeof(int) * ((n+1)/2) );
  if ( !ptrResult )
    return NULL;

  ptrCurr = ptrHead;
  /* Go through the first half of the list putting items into result
     buffer */
  tmp = (n+1) / 2;
  for ( i = 0; i < tmp; i++ ) {
    *(ptrResult + i) = ptrCurr->ItemValue;
    ptrCurr = ptrCurr->ptrNext;
  }
  i--;
  /* Continue going through the list, now multiplying items stored in
   * result buffer (in backwards order) by items in the second half of
   * the list. */
  if ( n & 0x01 ) {
    /* If number of items is odd, first square up the central item */
    *(ptrResult + i) *= *(ptrResult + i);
    i--;
  }
  for ( ; i >= 0; i-- ) {
    *(ptrResult + i) *= ptrCurr->ItemValue;
    ptrCurr = ptrCurr->ptrNext;
  }

  return ptrResult;
}



void PrintOut(itemptr ptrHead, int *ptrResult, int n)
{
  int i;

  puts("Original list:");
  while ( ptrHead ) {
    printf("%2d:  %3d\n", ptrHead->ItemKey, ptrHead->ItemValue);
    ptrHead = ptrHead->ptrNext;
  }
  puts("Resulting list:");
  for ( i = 0, n = (n+1) / 2; i < n; i++)
    printf("%2d:%5d\n", i, *(ptrResult + i));
}



void FreeListMem(itemptr ptrHead)
{
  itemptr ptrTemp;

  while ( ptrHead ) {
    ptrTemp = ptrHead->ptrNext;
    free((void *) ptrHead);
    ptrHead = ptrTemp;
  }
}



/*
 * Local variables:
 * compile-command: "gcc 3_05.c && ./a.out <<< 7"
 * End:
 */
