/*
 *  3.16.   ,  ,    L1 
 *  L2.
 */

#include <stdio.h>
#include <stdlib.h>

typedef struct _item {
  int ItemKey;
  int ItemValue;
  struct _item *ptrNext;
} item, *itemptr;

itemptr MakeUpList(int n, int *ptrItems);
itemptr lstlst(itemptr ptrHaystack, itemptr ptrNeedle);
void    PrintOut(itemptr ptrHead);
void    FreeListMem(itemptr ptrHead);


int ptrItems1[] = {  2, 5, 2, 7, 1, 8, 6, 4, 8, 9, 7 };
int ptrItems2[] = {           7, 1, 8                };

const char *strErrListMem = "Error allocating memory for a list.";
const char *strPrintFormat = "\nSecond list is %sa sublist of the first.\n";



int main()
{
  itemptr ptrHead1, ptrHead2;

  /* Make up two lists on the basis of the arrays */
  if ( !(ptrHead1 =
         MakeUpList(sizeof(ptrItems1)/sizeof(ptrItems1[0]), ptrItems1)) ) {
    puts(strErrListMem);
    return -1;
  }
  if ( !(ptrHead2 =
         MakeUpList(sizeof(ptrItems2)/sizeof(ptrItems2[0]), ptrItems2)) ) {
    puts(strErrListMem);
    FreeListMem(ptrHead1);
    return -1;
  }

  /* Print out the lists */
  puts("\n List 1:");
  PrintOut(ptrHead1);
  puts("\n List 2:");
  PrintOut(ptrHead2);

  /* See if the second list is a sublist of the first */
  printf(strPrintFormat, lstlst(ptrHead1, ptrHead2) ? "": "not ");

  /* Free the memory taken up by the lists */
  FreeListMem(ptrHead1);
  FreeListMem(ptrHead2);

  /* Bail out */
  return 0;
}



itemptr lstlst(itemptr ptrHaystack, itemptr ptrNeedle)
{
  int iNeedleFirstItem;
  itemptr ptrNeedleCurr, ptrHaystackCurr, ptrSublist;

  if ( !ptrNeedle )
    return NULL;

  /* To avoid dereferencing pointer to needle head in each loop, we
   * store first needle item in a local variable */
  ptrSublist = NULL;
  iNeedleFirstItem = ptrNeedle->ItemValue;
  while ( ptrHaystack ) {
    if ( ptrHaystack->ItemValue == iNeedleFirstItem ) {
      ptrNeedleCurr = ptrNeedle->ptrNext;
      ptrHaystackCurr = ptrHaystack->ptrNext;
      while ( ptrNeedleCurr &&
              ptrHaystackCurr &&
              ptrNeedleCurr->ItemValue == ptrHaystackCurr->ItemValue ) {
        ptrHaystackCurr = ptrHaystackCurr->ptrNext;
        ptrNeedleCurr = ptrNeedleCurr->ptrNext;
      }
      if ( !ptrNeedleCurr ) {
        ptrSublist = ptrHaystack;
        break;
      }
    }
    ptrHaystack = ptrHaystack->ptrNext;
  }

  return ptrSublist;
}



itemptr MakeUpList(int n, int *ptrItems)
{
  int i;
  itemptr ptrHead, ptrPrev, ptrCurr;

  ptrHead = (itemptr) malloc(sizeof(item));
  if ( !ptrHead )
    return NULL;
  ptrHead->ItemKey = 0;
  ptrHead->ItemValue = *ptrItems;
  ptrPrev = ptrHead;

  for ( i = 1; i < n; i++ ) {
    ptrCurr = (itemptr) malloc(sizeof(item));
    if ( !ptrCurr ) {
      ptrPrev->ptrNext = NULL;
      FreeListMem(ptrHead);
      return NULL;
    }
    ptrCurr->ItemKey = i;
    ptrCurr->ItemValue = *(ptrItems + i);
    ptrPrev->ptrNext = ptrCurr;
    ptrPrev = ptrCurr;
  }
  ptrPrev->ptrNext = NULL;

  return ptrHead;
}



void PrintOut(itemptr ptrHead)
{
  while ( ptrHead ) {
    printf("%2d:%5d\n", ptrHead->ItemKey, ptrHead->ItemValue);
    ptrHead = ptrHead->ptrNext;
  }
}



void FreeListMem(itemptr ptrHead)
{
  itemptr ptrTemp;

  while ( ptrHead ) {
    ptrTemp = ptrHead->ptrNext;
    free((void *) ptrHead);
    ptrHead = ptrTemp;
  }
}



/*
 * Local variables:
 * compile-command: "gcc 3_16.c && ./a.out"
 * End:
 */
