/*
 *  4.5.    ,    
 * ,  ,   .
 */

#include <stdio.h>
#include <stdlib.h>

typedef struct _litem {
  char Key;
  char Value;
  struct _litem *ptrNext;
  struct _litem *ptrPrev;
} litem, *litemptr;

litemptr MakeUpList();
litemptr lstlst(litemptr ptrHead, litemptr ptrCurr, char c);
void     FreeListMem(litemptr ptrHead);


const char *strErrListMem ="error: cannot allocate memory for the list\n";
const char *strErrOpenFile="error: cannot open file `%s'\n";


int main(int argc, char **argv)
{
  litemptr ptrHead, ptrCurr;
  char c;

  /* If user provided filename, try to redirect stdin to that file */
  if ( argc > 1 ) {
    if ( !freopen(*(argv+1), "r", stdin) ) {
      fprintf(stderr, strErrOpenFile, *(argv+1));
      return -1;
    }
  }

  /* Make up the list with character codes */
  if ( !(ptrHead = MakeUpList()) ) {
    fputs(strErrListMem, stderr);
    return -1;
  }

  ptrCurr = ptrHead;
  while ( (c=getchar()) != EOF ) {
    ptrCurr = lstlst(ptrHead, ptrCurr, c);
    putchar(ptrCurr->Value);
  }

  /* Free the list memory */
  FreeListMem(ptrHead);

  /* Bail out */
  return 0;
}


litemptr MakeUpList()
{
  int i;
  litemptr ptrHead, ptrPrev, ptrCurr;

  ptrHead = (litemptr) malloc(sizeof(litem));
  if ( !ptrHead )
    return NULL;
  ptrHead->Key   = 0;
  ptrHead->Value = 1;
  ptrHead->ptrPrev   = NULL;
  ptrPrev = ptrHead;

  for ( i = 1; i < 256; i++ ) {
    ptrCurr = (litemptr) malloc(sizeof(litem));
    if ( !ptrCurr ) {
      ptrPrev->ptrNext = NULL;
      FreeListMem(ptrHead);
      return NULL;
    }
    ptrCurr->Key = i;
    ptrCurr->Value = (i + 1) % 256;
    ptrCurr->ptrPrev = ptrPrev;
    ptrPrev->ptrNext = ptrCurr;
    ptrPrev = ptrCurr;
  }
  ptrPrev->ptrNext = NULL;

  return ptrHead;
}


litemptr lstlst(litemptr ptrHead, litemptr ptrCurr, char c)
{
  int dist;

  if ( !ptrHead )
    return NULL;
  if ( !ptrCurr )
    ptrCurr = ptrHead;

  /* Start searching either from the head of the list or from current hub,
   * whichever is closer to the character's hub */
  dist = c - ptrCurr->Key;
  if ( dist < 0 )
    dist = -dist;
  if ( c < dist )
    ptrCurr = ptrHead;

  if ( c < ptrCurr->Key ) /* Search backward */
    while ( ptrCurr && ptrCurr->Key != c )
      ptrCurr = ptrCurr->ptrPrev;
  else
    while ( ptrCurr && ptrCurr->Key != c )
      ptrCurr = ptrCurr->ptrNext; /* Search forward */

  return ptrCurr;
}


void FreeListMem(litemptr ptrHead)
{
  litemptr ptrTemp;

  while ( ptrHead ) {
    ptrTemp = ptrHead->ptrNext;
    free((void *) ptrHead);
    ptrHead = ptrTemp;
  }
}



/*
 * Local variables:
 * compile-command: "gcc 4_05.c && ./a.out 4_05.in"
 * End:
 */
