/*
 *  4.16.   L      
 * .     ,    L,  
 *   ,   ,    ""
 * (     ).
 */

#include <stdio.h>
#include <stdlib.h>

typedef struct _litem {
  int Key;
  int Value;
  struct _litem *ptrPrev;
  struct _litem *ptrNext;
} litem, *litemptr;

litemptr MakeUpList();
void     PrintOutList(litemptr ptrHead);
void     FreeListMem(litemptr ptrHead);


const char *strErrList ="error: list construction failed\n";


int main()
{
  litemptr ptrHead, ptrCurr, ptrTemp;
  int removed;

  /* Make up the list */
  if ( !(ptrHead = MakeUpList()) ) {
    fputs(strErrList, stderr);
    return -1;
  }

  /* Go through the ring list, removing items with identical
   * neighbors; stop as soon as no item is removed throughout one
   * whole cycle */
  ptrCurr = ptrHead;
  do {
    removed = 0;
    do {
      if ( ptrCurr->ptrPrev->Value == ptrCurr->ptrNext->Value &&
           ptrCurr->ptrPrev != ptrCurr->ptrNext ) {
        /* Cut out current item from the list */
        ptrCurr->ptrPrev->ptrNext = ptrCurr->ptrNext;
        ptrCurr->ptrNext->ptrPrev = ptrCurr->ptrPrev;
        /* Free memory taken up by it */
        ptrTemp = ptrCurr->ptrNext;
        free( (void *) ptrCurr );
        /* If what we are deleting is list head, change ptrHead to
         * point to something in the list */
        if ( ptrCurr == ptrHead )
          ptrHead = ptrTemp;
        ptrCurr = ptrTemp;
        removed = -1;
      } else
        ptrCurr = ptrCurr->ptrNext;
    } while ( ptrCurr != ptrHead );
  } while ( removed );

  /* Print out the resulting list */
  puts("Squeezed list:");
  PrintOutList(ptrHead);

  /* Free the list memory */
  FreeListMem(ptrHead);

  /* Bail out */
  return 0;
}


litemptr MakeUpList()
{
  int i, val;
  litemptr ptrHead, ptrPrev, ptrCurr;

  fputs("Enter the list (Ctrl-D to finish):\n", stdout);

  i = 0;
  if ( scanf("%d", &val) != EOF ) {
    ptrHead = (litemptr) malloc(sizeof(litem));
    if ( !ptrHead )
      return NULL;
    ptrHead->Key   = i++;
    ptrHead->Value = val;
    ptrPrev = ptrHead;

    while ( scanf("%d", &val) != EOF ) {
      ptrCurr = (litemptr) malloc(sizeof(litem));
      if ( !ptrCurr ) {
        ptrPrev->ptrNext = NULL;
        FreeListMem(ptrHead);
        return NULL;
      }
      ptrCurr->Key   = i++;
      ptrCurr->Value = val;
      ptrCurr->ptrPrev   = ptrPrev;
      ptrPrev->ptrNext   = ptrCurr;
      ptrPrev = ptrCurr;
    }
  } else
    return NULL;

  if ( i < 1 ){
    ptrPrev->ptrNext = NULL;
    FreeListMem(ptrHead);
    return NULL;
  }

  ptrPrev->ptrNext = ptrHead;
  ptrHead->ptrPrev = ptrPrev;

  return ptrHead;
}


void PrintOutList(litemptr ptrHead)
{
  litemptr ptrCurr;

  if ( !ptrHead )
    return;

  ptrCurr = ptrHead;
  do {
    printf("%d ", ptrCurr->Value);
    ptrCurr = ptrCurr->ptrNext;
  } while ( ptrCurr && ptrCurr != ptrHead );
  puts("");
}

void FreeListMem(litemptr ptrHead)
{
  litemptr ptrTemp, ptrCurr;

  ptrCurr = ptrHead;
  while ( ptrCurr && ptrCurr != ptrHead ) {
    ptrTemp = ptrCurr->ptrNext;
    free((void *) ptrCurr);
    ptrCurr = ptrTemp;
  }
}



/*
 * Local variables:
 * compile-command: "gcc 4_16.c && ./a.out <<< \"12 53 12 43 12\""
 * End:
 */
