/*
 *  5.5.    ,   .
 */

#include <stdio.h>
#include <stdlib.h>

typedef struct _tnode {
  int Value;
  struct _tnode *ptrLeft;
  struct _tnode *ptrRight;
} tnode, *tnodeptr;

int  MakeUpTree(tnodeptr ptrRoot, int level);
int  CropTree(tnodeptr ptrRoot);
void PrintOutTree(tnodeptr ptrRoot);
void _PrintOutTree(tnodeptr ptrRoot, int level, char LR);
void FreeTreeMem(tnodeptr ptrRoot);
void SpaceOut(int level);


const char *strErrTree ="error: tree construction failed\n";


int main()
{
  tnodeptr ptrRoot, ptrCurr, ptrTemp;
  int removed;

  /* Seed the random number generator */
  srand(time(0));

  if ( !(ptrRoot = (tnodeptr) malloc(sizeof(tnode))) ) {
    fputs(strErrTree, stderr);
    return -1;
  }

  /* Make up the tree */
  if ( !MakeUpTree(ptrRoot, 0) ) {
    fputs(strErrTree, stderr);
    return -1;
  }

  if ( CropTree(ptrRoot) )
    ptrRoot = NULL;

  /* Print out the resulting tree */
  puts("\nCropped tree:");
  PrintOutTree(ptrRoot);

  /* Free the list memory */
  FreeTreeMem(ptrRoot);

  /* Bail out */
  return 0;
}


int MakeUpTree(tnodeptr ptrRoot, int level)
{
  int c, answer, i;

  if ( !ptrRoot )
    return 0;

  /* Make sure the values are sane */
  ptrRoot->Value = rand() / (RAND_MAX/10);

  /* Left branch */
  while ( -1 ) {
    SpaceOut(level);
    fprintf(stdout, "L%d, go left  (y/n)? ", level);
    answer = c = getchar();
    /* Discard remaining characters */
    while ( c != '\n' && c != EOF )
      c = getchar();

    if ( answer == 'y' ) {
      ptrRoot->ptrLeft = (tnodeptr) malloc( sizeof(tnode) );
      if ( !MakeUpTree(ptrRoot->ptrLeft, level+1) )
        return 0;
      break;
    } else if ( answer == 'n' )
      break;
  }

  /* Right branch */
  while ( -1 ) {
    SpaceOut(level);
    fprintf(stdout, "L%d, go right (y/n)? ", level);
    answer = c = getchar();
    /* Discard remaining characters */
    while ( c != '\n' && c != EOF )
      c = getchar();

    if ( answer == 'y' ) {
      ptrRoot->ptrRight = (tnodeptr) malloc( sizeof(tnode) );
      if ( !MakeUpTree(ptrRoot->ptrRight, level+1) )
        return 0;
      break;
    } else if ( answer == 'n' )
      break;
  }

  return -1;
}


/* Returns -1 if subtree starting with ptrRoot node was removed, 0 if
 * it remained intact */
int CropTree(tnodeptr ptrRoot)
{
  if ( !ptrRoot )
    return 0;

  /* Remove subtree starting with ptrRoot node if this node has only
   * one son */
  if ( (!ptrRoot->ptrLeft &&  ptrRoot->ptrRight) ||
       ( ptrRoot->ptrLeft && !ptrRoot->ptrRight) ) {
    FreeTreeMem(ptrRoot);
    return -1;
  }

  /* See if left or right subtrees should be removed, and if so,
   * cancel references to the removed subtrees from ptrRoot node */
  if ( CropTree(ptrRoot->ptrLeft) )
    ptrRoot->ptrLeft = NULL;
  if ( CropTree(ptrRoot->ptrRight) )
    ptrRoot->ptrRight = NULL;

  return 0;
}

void PrintOutTree(tnodeptr ptrRoot)
{
  _PrintOutTree(ptrRoot, 0, 'O');
}

void _PrintOutTree(tnodeptr ptrRoot, int level, char LR)
{
  if ( !ptrRoot )
    return;

  SpaceOut(level);
  printf("%c%d: %d\n", LR, level, ptrRoot->Value);
  if (ptrRoot->ptrLeft)
    _PrintOutTree(ptrRoot->ptrLeft, level+1, 'L');
  if (ptrRoot->ptrRight)
    _PrintOutTree(ptrRoot->ptrRight, level+1, 'R');
}

void FreeTreeMem(tnodeptr ptrRoot)
{
  if (!ptrRoot)
    return;

  FreeTreeMem(ptrRoot->ptrLeft);
  FreeTreeMem(ptrRoot->ptrRight);
  free((void *) ptrRoot);
}

void SpaceOut(int level)
{
  int i;

  for ( i = 0; i < level; i++ )
    putchar(' ');
}



/*
 * Local variables:
 * compile-command: "gcc 5_05.c"
 * End:
 */
