#ifndef _FP_HEADER_
#define _FP_HEADER_

#include <stdio.h>
#include <stdlib.h>

/* Node types */
#define NT_NUMBER               0
#define NT_OPERATION            1

/* Types of parenthesising for operands of unary operations (use only one of the
   operand parenthesising flags on a single operation) */
#define UNARY_PAREN_OP_ALWAYS   1 /* always parenthesise operand */
#define UNARY_PAREN_OP_NESTED   2 /* parenthesise operand only when operand
                                     contains nested operations */
#define UNARY_PAREN_OP_NEVER    4 /* never parenthesise operand */
/* Types of parenthesising for unary operations */
#define UNARY_PAREN_WHEN_RIGHT  8 /* parenthesise operation itself if it is a
                                     right-hand operand of a binary operation */

typedef struct _operation {
  char  code;     /* input code */
  int   ops;      /* number of operands (1 or 2) */
  char *prefix;   /* for binary operations this is operation's output symbol;
                     for unary operations this is output prefix */
  char *postfix;  /* for binary operations this should be NULL, for unary
                     operations this is output postfix */
  int   priority; /* for binary operations this is priority; for unary
                     operations this is bit field indicating when to
                     parenthasise this operation and its operand */
} operation, *operationptr;

extern const operation Operations[];

/* Operation ID's (offsets into Operations[]) */
#define ID_PLUS             0
#define ID_MINUS            1
#define ID_MULTIPLICATION   2
#define ID_DIVISION         3
#define ID_MODULO           4
#define ID_POWER            5
#define ID_ABS              6
#define ID_NEGATION         7
#define ID_SIGNUM           8
#define ID_FACTORIAL        9

typedef struct _tnode {
  int  type;                /* Node type (operation or number) */
  long value;               /* Node value (operation ID or number) */
  struct _tnode *ptrLeft;
  struct _tnode *ptrRight;
} tnode, *tnodeptr;


tnodeptr ParseLine(char *line, int lineno, int InteractiveMode);

tnodeptr MakeNode(int type, long value);
void     FreeTreeMem(tnodeptr ptrRoot);
void     PrintOutTree(tnodeptr ptrRoot);

char    *UnparseTree(tnodeptr ptrRoot);
char    *GetUnparseBuffer(void);
void     FreeUnparseBuffer(void);

int      Push(tnodeptr ptrNode);
int      Pop(tnodeptr *ptrNode);
void     FreeStackedTrees(void);
void     FreeStack(void);

extern const char *strErrOpenFile;
extern const char *strErrMem;
extern const char *strErrUnknownOperation;
extern const char *strErrUnexpectedCharInteractive;
extern const char *strErrUnexpectedCharNoninteractive;
extern const char *strErrParsingExpressionInteractive;
extern const char *strErrParsingExpressionNoninteractive;
extern const char *strErrUnparsingTree;


#endif /* _FP_HEADER_ */
