/*
 *  5.16.     ,   -
 *    .
 */

#include "fp.h"

#define LINE_BUFFER_LENGTH  1024

const operation Operations[] = {
  {'+',  2, " + ", NULL, 0},
  {'-',  2, " - ", NULL, 0},
  {'*',  2, " * ", NULL, 1},
  {'/',  2, " / ", NULL, 1},
  {'\%', 2, " % ", NULL, 1},
  {'^',  2, " ^ ", NULL, 3},
  {'$',  1, "|",   "|",  UNARY_PAREN_OP_NEVER},
  {'~',  1, "-",   NULL, UNARY_PAREN_OP_NESTED | UNARY_PAREN_WHEN_RIGHT},
  {'#',  1, "sgn", NULL, UNARY_PAREN_OP_ALWAYS},
  {'!',  1, NULL,  "!",  UNARY_PAREN_OP_NESTED}
};

const char *strGreeting           ="Enter formulas in reverse polish notation (Ctrl-D to quit):\n";
const char *strParsing            ="***Parsing formula:\n%s";
const char *strConstructedTree    ="***Constructed tree:\n";
const char *strUnparsedFormula    ="***Unparsed formula:\n";
const char *strErrOpenFile        ="fp: cannont open file `%s'\n";
const char *strErrMem             ="fp: not enough memory\n";
const char *strErrUnknownOperation="fp: unknown operation `%c'\n";
const char *strErrUnexpectedCharInteractive="fp: column %d--unexpected character `%c'\n";
const char *strErrUnexpectedCharNoninteractive="fp: line %d, col %d--unexpected character `%c'\n";
const char *strErrParsingExpressionInteractive="fp: unable to parse expression\n";
const char *strErrParsingExpressionNoninteractive="fp: unable to parse expression on line %d\n";
const char *strErrUnparsingTree   ="fp: unable to unparse formula tree\n";

static char *line;


int main(int argc, char **argv)
{
  int InteractiveMode, lineno;
  char *str;
  tnodeptr ptrRoot;

  /* If user provided filename, try to redirect stdin to that file */
  if ( argc > 1 ) {
    if ( !freopen(*(argv+1), "r", stdin) ) {
      fprintf(stderr, strErrOpenFile, *(argv+1));
      return -1;
    }
    InteractiveMode = 0;
  } else {
    fputs(strGreeting, stdout);
    InteractiveMode = -1;
  }

  /* Allocate line buffer to read input into */
  if ( !(line = malloc(LINE_BUFFER_LENGTH)) ) {
    fputs(strErrMem, stderr);
    return -1;
  }

  /* Main loop */
  lineno = 1;
  while ( fgets(line, LINE_BUFFER_LENGTH, stdin) ) {
    if ( !InteractiveMode )
      fprintf(stdout, strParsing, line);
    /* Construct tree from the formula */
    ptrRoot = ParseLine(line, lineno++, InteractiveMode);
    if ( ptrRoot ) {
      /* Print out the constructed tree */
      fputs(strConstructedTree, stdout);
      PrintOutTree(ptrRoot);
      /* Unparse tree back into formula and print it out */
      fputs(strUnparsedFormula, stdout);
      if ( str = UnparseTree(ptrRoot) )
        puts(str);
      FreeUnparseBuffer();
    }
    putchar('\n');
  }

  FreeStack();

  return 0;
}



/*
 * Local variables:
 * compile-command: "make -k all && echo && ./fp fp.in"
 * End:
 */
