#include "fp.h"

static void ReportParsingError(int InteractiveMode, int lineno);
static void ReportUnexpectedChar(int InteractiveMode, char c, int lineno, int colno);

tnodeptr ParseLine(char *line, int lineno, int InteractiveMode)
{
  char *ptr;
  long op1, op2;
  int  sgn, BailOut, colno;
  tnodeptr ptrNode1, ptrNode2;

  ptr = line;

  BailOut = 0;
  colno = 0;
  while ( *ptr && !BailOut ) {
    colno++;
    if ( isalpha(*ptr) ) {
      ReportUnexpectedChar(InteractiveMode, *ptr, lineno, colno);
      BailOut = -1;

    } else if ( isspace(*ptr) ) {
      /* Skip spaces */
      ptr++;

    } else if ( isdigit(*ptr) || *ptr == '_' ) {
      /* Parse out the number */
      if ( *ptr == '_' ) { /* Minus sign */
        sgn = -1;
        ptr++;
      } else
        sgn = 0;
      op1 = strtol(ptr, &ptr, 0);
      if ( sgn )
        op1 = -op1;
      /* Construct the node and push it onto stack */
      if ( !(ptrNode1 = MakeNode(NT_NUMBER, op1)) || !Push(ptrNode1) )
        BailOut = -1;

    } else {
      int OpID;

      /* Parse out the operation */
      switch ( *ptr ) {
      case '+':
        OpID = ID_PLUS;
        break;
      case '-':
        OpID = ID_MINUS;
        break;
      case '*':
        OpID = ID_MULTIPLICATION;
        break;
      case '/':
        OpID = ID_DIVISION;
        break;
      case '\%': /* modulo */
        OpID = ID_MODULO;
        break;
      case '$': /* absolute value */
        OpID = ID_ABS;
        break;
      case '~': /* negate */
        OpID = ID_NEGATION;
        break;
      case '^': /* flower power */
        OpID = ID_POWER;
        break;
      case '#': /* signum */
        OpID = ID_SIGNUM;
        break;
      case '!': /* factorial */
        OpID = ID_FACTORIAL;
        break;
      default:
        ReportUnexpectedChar(InteractiveMode, *ptr, lineno, colno);
        BailOut = -1;
        break;
      }

      if ( !BailOut ) {
        /* Create node for this operation */
        if ( ptrNode1 = MakeNode(NT_OPERATION, OpID) )
          /* Pop top node which should be a second operand */
          if ( Pop(&ptrNode1->ptrRight) ) {
            /* Pop next node which should be first operand */
            if ( (Operations+OpID)->ops == 2 ) {
              if ( Pop(&ptrNode1->ptrLeft) ) {
              } else {
                ReportParsingError(InteractiveMode, lineno);
                free( (void *) ptrNode1->ptrRight );
                free( (void *) ptrNode1 );
                BailOut = -1;
              }
            }
          } else {
            ReportParsingError(InteractiveMode, lineno);
            free( (void *) ptrNode1 );
            BailOut = -1;
          }
        else {
          BailOut = -1;
        }
      }

      /* Push constructed subtree back onto stack */
      if ( !BailOut ) {
        if ( !Push(ptrNode1) ) {
          FreeTreeMem(ptrNode1);
          BailOut = -1;
        }
      }

      ptr++;
    }
  }

  if ( BailOut ) {
    FreeStackedTrees();
    return NULL;
  }

  /* Pop the stack--if expression is correct (or this program is not
   * buggy!) the stack will contain exactly one node which is the root
   * we are about to return */
  if ( !Pop(&ptrNode1) ) {
    ReportParsingError(InteractiveMode, lineno);
    return NULL;
  }

  /* Make sure stack contains no other nodes pushed onto it */
  if ( Pop(&ptrNode2) ) {
    ReportParsingError(InteractiveMode, lineno);
    free( (void *) ptrNode2 );
    FreeStackedTrees();
    return NULL;
  }

  return ptrNode1;
}

static void ReportParsingError(int InteractiveMode, int lineno)
{
  if ( InteractiveMode )
    fputs(strErrParsingExpressionInteractive, stderr);
  else
    fprintf(stderr, strErrParsingExpressionNoninteractive, lineno);

  return;
}

static void ReportUnexpectedChar(int InteractiveMode, char c, int lineno, int colno)
{
  if ( InteractiveMode )
    fprintf(stderr, strErrUnexpectedCharInteractive, colno, c);
  else
    fprintf(stderr, strErrUnexpectedCharNoninteractive, lineno, colno, c);

  return;
}




/*
 * Local variables:
 * compile-command: "make -k all && echo && ./fp fp.in"
 * End:
 */
