#include "fp.h"

#define STACK_MEM_CHUNK     16

static tnodeptr *Stack = NULL;
static int StackTop = -1;
static int StackLen = 0;

/* Returns length of stack on success, 0 on failure to grow stack */
int Push(tnodeptr ptrNode)
{
  tnodeptr *ptr;

  /* Grow stack if necessary */
  if ( ++StackTop >= StackLen ) {
    StackLen += STACK_MEM_CHUNK;
    if ( !(ptr = (tnodeptr *) realloc(Stack, StackLen * sizeof(*Stack))) ) {
      StackLen -= STACK_MEM_CHUNK;
      fputs(strErrMem, stderr);
      return 0;
    }
    Stack = ptr;
  }
  /* Push the value onto stack */
  *(Stack + StackTop) = ptrNode;

  return StackTop + 1;
}

/* Returns -1 on success, 0 if stack is empty */
int Pop(tnodeptr *ptrNode)
{
  if ( StackTop >= 0 ) {
    *ptrNode = *(Stack + StackTop--);
    return -1;
  }

  return 0;
}

void FreeStackedTrees(void)
{
  tnodeptr *ptrCurr;

  if ( !Stack )
    return;

  for ( ptrCurr = Stack + StackTop; ptrCurr >= Stack; ptrCurr-- )
    FreeTreeMem(*ptrCurr);

  StackTop = -1;

  return;
}

void FreeStack(void)
{
  FreeStackedTrees();
  free(Stack);
  Stack = NULL;
  StackLen = 0;
  StackTop = -1;
}




/*
 * Local variables:
 * compile-command: "make -k all && echo && ./fp fp.in"
 * End:
 */
