/*
 * Recursive algorithm for calculating determinant of a matrix
 */
#include <stdio.h>
#include <stdlib.h>


const char *strErrOpenFile      = "error: cannont open file `%s'\n";
const char *strErrMem           = "error: not enough memory\n";
const char *strErrUnexpectedEOF = "error: unexpected end of input\n";

int      n;
double **matrix;


double determ(int l, double **matrix);


int main(int argc, char **argv)
{
  int i, j;
  int interact = -1;

  /* If user provided filename, try to redirect stdin to that file */
  if ( argc > 1 ) {
    if ( !freopen(*(argv+1), "r", stdin) ) {
      fprintf(stderr, strErrOpenFile, *(argv+1));
      return -1;
    }
    interact = 0;
  }

  /* Get matrix dimensions from user */
  if ( interact )
    fputs("Enter matrix dimension: ", stdout);
  if ( scanf("%d", &n) == EOF ) {
    fputs(strErrUnexpectedEOF, stderr);
    return -1;
  }

  /* Memory allocations */
  if ( !( matrix = (double **) malloc( sizeof(double *) * n ) ) ) {
    fputs(strErrMem, stderr);
    return -1;
  }

  for ( i = 0; i < n; i++ ) {
    if ( !( *(matrix + i) = (double *) malloc( sizeof(double) * n) ) ) {
      fputs(strErrMem, stderr);
      return -1;
    }
  }

  /* Get matrix elements from user */
  if ( interact )
    fputs("Enter matrix elements: ", stdout);

  for ( i = 0; i < n; i++ )
    for ( j = 0; j < n; j++ )
      if ( scanf("%lf", *(matrix + i) + j) == EOF ) {
        fputs(strErrUnexpectedEOF, stderr);
        return -1;
      }

  /* Print out the matrix */
  if ( interact ) {
    int n_minus_one = n - 1;
    for ( i = 0; i < n; i++ ) {
      for ( j = 0; j < n_minus_one; j++ )
        fprintf(stdout, "%6.2lf ", *(*(matrix + i) + j));
      fprintf(stdout, "%6.2lf", *(*(matrix + i) + j));
      fputc('\n', stdout);
    }
    putchar('\n');
  }

  /* Calculate and print determinant */
  fprintf(stdout, "%lf\n", determ(0, matrix));

  /* Free the mallocs */
  for ( i = 0; i < n; i++ )
    free((void *) *(matrix + i));
  free((void *) matrix);

  return 0;
}


/*
 * Calculates determinant by adding up algebraic complements for the first
 * column.  Recursively calls itself to calculate determinants for the algebraic
 * complements.
 */
double determ(int l, double **matrix)
{
  int      i;
  double  *ptrTemp;
  double   sum        = 0;
  int      l_plus_one = l + 1;
  int      n_minus_l  = n - l;
  double **ptrNext    = matrix + 1;

  if ( l_plus_one == n )
    return *(*matrix + l);

  sum += *(*matrix + l) * determ(l_plus_one, ptrNext);
  for ( i = 1; i < n_minus_l; i++ ) {
    /* Swap i-th line with the first line */
    ptrTemp = *matrix;
    *matrix = *(matrix + i);
    *(matrix + i) = ptrTemp;

    /* Swapping first line with i-th line automagically makes sign of this
       complement a minus:  number of transpositions required to exchange first
       line with i-th line is even for even i's (minus remains) and odd for odd
       i's (plus is negated) */
    sum -= *(*matrix + l) * determ(l_plus_one, ptrNext);

    /* Swap first line back into place */
    ptrTemp = *matrix;
    *matrix = *(matrix + i);
    *(matrix + i) = ptrTemp;
  }

  return sum;
}



/*
 * Local variables:
 * compile-command: "gcc -Wall recursive_determinant.c && echo && ./a.out <<< \"3 1 -1 0 -1 2 3 -1 0 1\""
 * End:
 */
